import { Body, Controller, Logger } from "@nestjs/common";

import { MessagePattern, RpcException } from "@nestjs/microservices";
import {
  BasicMessageEvent,
  CloudEventDto,
  CreateCredentialDefinitionRequestDto,
  CreateSchemaRequestDto,
  CredentialEvent,
  GatewayAcceptedResponseDto,
  IssueCredentialRequestDto,
  MakeBasicMessageRequestDto,
  makeEvent,
  SchemaEvent,
} from "@ocm-engine/dtos";
import { ProducerService } from "@ocm-engine/nats";

@Controller()
export class AppController {
  private readonly logger: Logger = new Logger(AppController.name);

  constructor(private readonly producerService: ProducerService) {}

  @MessagePattern("schemas")
  async create(
    @Body()
    payload: {
      data: null | CreateSchemaRequestDto | CreateSchemaRequestDto;
      type: SchemaEvent;
      source: string;
    },
  ): Promise<GatewayAcceptedResponseDto> {
    this.logger.debug(JSON.stringify(payload, null, 2));

    try {
      const event = makeEvent(payload);
      this.logger.debug(JSON.stringify(event, null, 2));
      await this.producerService.publish<typeof payload.data>(
        payload.type,
        event as CloudEventDto<typeof payload.data>,
      );

      const response = new GatewayAcceptedResponseDto();
      response.id = event.id;

      return response;
    } catch (e) {
      this.logger.debug(JSON.stringify(e, null, 2));
      if (e instanceof Error) {
        throw new RpcException(e.message);
      }

      throw new RpcException("Internal server error");
    }
  }

  @MessagePattern("credentials")
  async createCredential(
    @Body()
    payload: {
      data:
        | null
        | CreateCredentialDefinitionRequestDto
        | IssueCredentialRequestDto;
      type: CredentialEvent;
      source: string;
    },
  ): Promise<GatewayAcceptedResponseDto> {
    this.logger.debug(JSON.stringify(payload, null, 2));

    try {
      const event = makeEvent(payload);
      this.logger.debug(JSON.stringify(event, null, 2));
      await this.producerService.publish<typeof payload.data>(
        payload.type,
        event as CloudEventDto<typeof payload.data>,
      );

      const response = new GatewayAcceptedResponseDto();
      response.id = event.id;

      return response;
    } catch (e) {
      this.logger.debug(JSON.stringify(e, null, 2));
      if (e instanceof Error) {
        throw new RpcException(e.message);
      }

      throw new RpcException("Internal server error");
    }
  }

  @MessagePattern("messages")
  async sendMessage(
    @Body()
    payload: {
      data: MakeBasicMessageRequestDto;
      type: BasicMessageEvent;
      source: string;
    },
  ): Promise<GatewayAcceptedResponseDto> {
    this.logger.debug(JSON.stringify(payload, null, 2));

    try {
      const event = makeEvent(payload);
      this.logger.debug(JSON.stringify(event, null, 2));
      await this.producerService.publish<typeof payload.data>(
        payload.type,
        event as CloudEventDto<typeof payload.data>,
      );

      const response = new GatewayAcceptedResponseDto();
      response.id = event.id;

      return response;
    } catch (e) {
      this.logger.debug(JSON.stringify(e, null, 2));
      if (e instanceof Error) {
        throw new RpcException(e.message);
      }

      throw new RpcException("Internal server error");
    }
  }
}
