import {makeAutoObservable, runInAction} from "mobx";
import api from "@dashboard/api";
import {toast} from "react-toastify";
import {ProofRecordDto} from "@dashboard/api/agent-client";


class ProofListStore {
  public loading = false;
  public items: ProofRecordDto[] = [];

  public statusChangeId: string | null = null;

  constructor() {
    makeAutoObservable(this);
  }

  public async loadAll(connectionId: string | null) {
    runInAction(() => this.loading = true);
    try {
      const items = await api.proofsQuery({
        connectionId: connectionId || undefined,
      });
      runInAction(() => {
        this.items = items;
        this.loading = false;
      });
    } catch (e: any) {
      toast(e.message);
      console.error(e);
      runInAction(() => this.loading = false);
    }
  }

  public async accept(proofRecordId: string) {
    runInAction(() => {
      this.statusChangeId = proofRecordId;
    });
    try {
      const credRecord = await api.acceptProof({
        proofId: proofRecordId,
        proofUrl: ""
      });
      runInAction(() => {
        const index = this.items.findIndex(p => p.id === credRecord.id);
        if (index >= 0) {
          this.items[index] = credRecord;
        }
        this.statusChangeId = null;
      });
    } catch (e: any) {
      toast(e.message);
      console.error(e);
      runInAction(() => this.loading = false);
    }
  }

  public async decline(proofRecordId: string) {
    runInAction(() => {
      this.statusChangeId = proofRecordId;
    });
    try {
      const credRecord = await api.declineProofRequest(proofRecordId);
      runInAction(() => {
        const index = this.items.findIndex(p => p.id === credRecord.id);
        if (index >= 0) {
          this.items[index] = credRecord;
        }
        this.statusChangeId = null;
      });
    } catch (e: any) {
      toast(e.message);
      console.error(e);
      runInAction(() => this.loading = false);
    }
  }

  public async remove(proofRecordId: string) {
    runInAction(() => {
      this.statusChangeId = proofRecordId;
    });
    try {
      await api.deleteProofById(proofRecordId);
      runInAction(() => {
        const index = this.items.findIndex(p => p.id === proofRecordId);
        if (index >= 0) {
          this.items.splice(index, 1);
        }
        this.statusChangeId = null;
      });
    } catch (e: any) {
      toast(e.message);
      console.error(e);
      runInAction(() => this.loading = false);
    }
  }
}

export type { ProofListStore };

export default ProofListStore;
