import {makeAutoObservable, runInAction} from "mobx";
import api from "@dashboard/api";
import {
  CreddefRecordDto,
  ProofRecordDto,
  SchemaRecordDto
} from "@dashboard/api/agent-client";
import {toast} from "react-toastify";
import {v4} from "uuid";


class LoginPageStore {
  public loading = false;
  public id = v4();
  public schemaId = 'did:indy:bcovrin:test:FcZ4aS728oRGVdNUKVaECP/anoncreds/v0/SCHEMA/Identity/1.0';
  public schema: SchemaRecordDto | null = null;
  public creddef: CreddefRecordDto | null = null;
  public shortProofUrl: string | null = null;
  public proofUrl: string | null = null;
  public proofRecord: ProofRecordDto | null = null;

  constructor() {
    makeAutoObservable(this);
  }

  public async initialize() {
    runInAction(() => this.loading = true);
    try {
      const schema = await api.getSchemaById({ id: this.schemaId });
      const credDefs = await api.credentialDefinitions();
      let requiredCredDef = credDefs.find((credDef) => credDef.schemaId === this.schemaId);
      if (!requiredCredDef) {
        requiredCredDef = await api.createCredentialDefinition({ schemaId: this.schemaId, tag: 'Identity' });
      }

      const { shortProofUrl, proofUrl, proofRecord } = await api.issueProof({
        attributes: [
          {
            attributeName: 'internal-id',
            credentialDefinitionId: requiredCredDef.id,
            schemaId: this.schemaId
          },
          {
            attributeName: 'name',
            credentialDefinitionId: requiredCredDef.id,
            schemaId: this.schemaId
          }
        ]
      })
      runInAction(() => {
        this.schema = schema;
        this.creddef = requiredCredDef!;
        this.shortProofUrl = shortProofUrl;
        this.proofUrl = proofUrl;
        this.proofRecord = proofRecord;
        this.loading = false;
      });
    } catch (e: any) {
      console.error(e);
      toast.error(e.message);
      runInAction(() => this.loading = false);
    }
  }
}

export type { LoginPageStore };

export default LoginPageStore;
