import {
  Body,
  Controller,
  Get,
  Param,
  Post,
  Delete,
  UseFilters,
} from "@nestjs/common";
import { AllExceptionsHandler } from "../exception.handler";
import { AttestationManagerClient } from "@ocm-engine/clients";
import {
  CreateCredentialDefinitionRequestDto,
  CreateSchemaRequestDto,
  CredentialFilterDto,
  IssueCredentialRequestDto,
  GatewayAcceptedResponseDto,
  CRED_DEF_CREATE,
  CRED_DEF_LIST,
  CRED_DEF_GET,
  CRED_ISSUE,
  CRED_LIST,
  CRED_QUERY,
  CRED_GET,
  CRED_OFFER_ACCEPT,
  CRED_OFFER_DECLINE,
  CRED_DELETE,
  CRED_OFFER_LIST,
  MakeBasicMessageRequestDto,
  MessageFilterDto,
  MESSAGES_MAKE,
  MESSAGES_QUERY,
  MESSAGES_DELETE,
  SCHEMA_CREATE,
  SCHEMA_GET,
  SCHEMA_LIST,
  IdReqDto,
} from "@ocm-engine/dtos";
import {
  ApiBadRequestResponse,
  ApiInternalServerErrorResponse,
  ApiOperation,
  ApiResponse,
} from "@nestjs/swagger";

@UseFilters(AllExceptionsHandler)
@Controller("v1")
export class AttestationController {
  constructor(private readonly amClient: AttestationManagerClient) {}

  @Post("/schemas")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },

            message: {
              type: "array",
              example: ["name must be a string", "name should not be empty"],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Create schema",
    description:
      "Method will create schema. The id of the response will be matched when you receive event from the websocket",
    tags: ["Schema"],
  })
  async createSchema(@Body() schemaDto: CreateSchemaRequestDto): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload<CreateSchemaRequestDto>({
      pattern: "schemas",
      payload: {
        source: "/schema",
        data: schemaDto,
        type: SCHEMA_CREATE,
      },
    });
  }

  @Post("/schemas-by-id")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },

            message: {
              type: "array",
              example: [
                "schemaId must be a string",
                "schemaId should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Get schema by id",
    description:
      "Method will fetch specific schema or return null. The id of the response will be matched when you receive event from the websocket",
    tags: ["Schema"],
  })
  async getSchemaById(@Body() schemaRequestDto: IdReqDto): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload<IdReqDto>({
      pattern: "schemas",
      payload: {
        source: "/schemas-by-id",
        data: schemaRequestDto,
        type: SCHEMA_GET,
      },
    });
  }

  @Get("/schemas")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "List all schemas",
    description:
      "Method will fetch all schemas. The id of the response will be matched when you receive event from the websocket",
    tags: ["Schema"],
  })
  async listSchema(): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload<null>({
      pattern: "schemas",
      payload: {
        source: "/schema",
        data: null,
        type: SCHEMA_LIST,
      },
    });
  }

  @Get("/credentials/definitions")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "List all created credential definitions",
    description:
      "Method list credential definition no filters applied. The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials"],
  })
  async credentialDefinitions(): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload({
      pattern: "credentials",
      payload: {
        source: "credentials/definitions",
        data: null,
        type: CRED_DEF_LIST,
      },
    });
  }

  @Post("credentials/definition-by-id")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },

            message: {
              type: "array",
              example: [
                "id must be a string",
                "id should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Get credential definition by id",
    description:
      "Method will fetch specific credential definition or return null. The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials"],
  })
  async getCredentialDefinitionsById(@Body() credDefRequestDto: IdReqDto): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload<IdReqDto>({
      pattern: "credentials",
      payload: {
        source: "/credentials/definition-by-id",
        data: credDefRequestDto,
        type: CRED_DEF_GET,
      },
    });
  }

  @Post("credentials/definition")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },

            message: {
              type: "array",
              example: [
                "schemaId must be a string",
                "schemaId should not be empty",
                "tag must be a string",
                "tag should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Create credential definition",
    description:
      "Method create credential definition. The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials"],
  })
  async createCredentialDefinition(
    @Body() credentialDefinitionDto: CreateCredentialDefinitionRequestDto,
  ): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload({
      pattern: "credentials",
      payload: {
        source: "/credential/definition",
        data: credentialDefinitionDto,
        type: CRED_DEF_CREATE,
      },
    });
  }

  @Post("credentials/issue")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },

            message: {
              type: "array",
              example: [
                "connectionId must be a string",
                "connectionId should not be empty",
                "credentialDefinitionId must be a string",
                "credentialDefinitionId should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Issue credential",
    description:
      "Method issue credential, it will create an offer and send it to specified receiver (connectionId). The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials"],
  })
  async issueCredential(@Body() issueCredentialDto: IssueCredentialRequestDto): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload({
      pattern: "credentials",
      payload: {
        source: "credential/issue",
        data: issueCredentialDto,
        type: CRED_ISSUE,
      },
    });
  }

  @Get("/credentials")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "List all credential",
    description:
      "Method list credential definition no filters applied. The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials"],
  })
  async credentials(): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload({
      pattern: "credentials",
      payload: {
        source: "credentials",
        data: null,
        type: CRED_LIST,
      },
    });
  }

  @Post("/credentials/query")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },
            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Query credentials based on filters",
    description:
      "This method queries credential definitions based on provided filters. The id of the response will be matched when you receive an event from the websocket.",
    tags: ["Credentials"],
  })
  async credentialsQuery(@Body() credentialFilterDto: CredentialFilterDto): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload<CredentialFilterDto>({
      pattern: "credentials",
      payload: {
        source: "credentials",
        data: credentialFilterDto,
        type: CRED_QUERY,
      },
    });
  }

  @Get("/credentials/:id")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },
            message: {
              type: "array",
              example: [
                "id must be a string",
                "id should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },
            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Get credential by ID",
    description:
      "Method retrieves a specific credential based on the provided ID or returns null if not found. The ID of the response will be matched when you receive an event from the websocket",
    tags: ["Credentials"],
  })
  async getCredentialById(@Param("id") credentialId: string): Promise<GatewayAcceptedResponseDto> {
    const data = new IdReqDto();
    data.id = credentialId;

    return this.amClient.sendPayload<IdReqDto>({
      pattern: "credentials",
      payload: {
        source: "/credentials/:id",
        data,
        type: CRED_GET,
      },
    });
  }

  @Get("/credentials/offers")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "List unaccepted credential offers",
    description:
      "Method list offers that are received, but not accepted. The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials Offers"],
  })
  async getCredentialOffers(): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload({
      pattern: "credentials",
      payload: {
        source: "credentials/offers",
        data: null,
        type: CRED_OFFER_LIST,
      },
    });
  }

  @Post("/credentials/offers/:credential_record_id/accept")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Accept credential offers",
    description:
      "Method list accept credential offer. The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials Offers"],
  })
  async acceptCredential(@Param("credential_record_id") credentialRecordId: string): Promise<GatewayAcceptedResponseDto> {
    const data = new IdReqDto();
    data.id = credentialRecordId;
    return this.amClient.sendPayload({
      pattern: "credentials",
      payload: {
        source: "credential/:id/accept",
        data,
        type: CRED_OFFER_ACCEPT,
      },
    });
  }

  @Post("/credentials/offers/:credential_record_id/decline")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },
            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Decline credential offers",
    description:
      "Method to decline a credential offer. The id of the response will be matched when you receive an event from the websocket.",
    tags: ["Credentials Offers"],
  })
  async declineCredential(@Param("credential_record_id") credentialRecordId: string): Promise<GatewayAcceptedResponseDto> {
    const data = new IdReqDto();
    data.id = credentialRecordId;

    return this.amClient.sendPayload({
      pattern: "credentials",
      payload: {
        source: "credential/:id/decline",
        data,
        type: CRED_OFFER_DECLINE,
      },
    });
  }

  @Delete("/credentials/:id")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },
            message: {
              type: "array",
              example: [
                "credentialId must be a string",
                "credentialId should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },
            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Delete a credential by ID",
    description:
      "Method deletes a specific credential based on provided ID. The id of the response will be matched when you receive an event from the websocket",
    tags: ["Credentials"],
  })
  async deleteCredentialById(@Param("id") credentialId: string): Promise<GatewayAcceptedResponseDto> {
    const data = new IdReqDto();
    data.id = credentialId;

    return this.amClient.sendPayload<IdReqDto>({
      pattern: "credentials",
      payload: {
        source: "/credentials/:id",
        data,
        type: CRED_DELETE,
      },
    });
  }

  @Post("/messages")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },

            message: {
              type: "array",
              example: [
                "connectionId must be a string",
                "connectionId should not be empty",
                "message must be a string",
                "message should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },

            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Send basic message",
    description:
      "Method will send basic message to a connection. The id of the response will be matched when you receive event from the websocket",
    tags: ["Credentials Offers"],
  })
  async sendMessage(@Body() message: MakeBasicMessageRequestDto): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload({
      pattern: "messages",
      payload: {
        source: "messages",
        data: message,
        type: MESSAGES_MAKE,
      },
    });
  }

  @Post("/messages/query")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },
            message: {
              type: "array",
              example: [
                "message must be a string",
                "message should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },
            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Query messages based on filters",
    description:
      "Method will query and return messages based on the provided filters. The id of the response will be matched when you receive event from the websocket",
    tags: ["Messages"],
  })
  async queryMessages(@Body() filter: MessageFilterDto): Promise<GatewayAcceptedResponseDto> {
    return this.amClient.sendPayload<MessageFilterDto>({
      pattern: "messages",
      payload: {
        source: "messages",
        data: filter,
        type: MESSAGES_QUERY,
      },
    });
  }

  @Delete("/messages/:id")
  @ApiResponse({
    status: 200,
    description:
      "Request is accepted for execution, the response id will match the event id received from the web socket",
    type: GatewayAcceptedResponseDto,
  })
  @ApiBadRequestResponse({
    description: "Validation error",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 400,
            },
            message: {
              type: "array",
              example: [
                "messageId must be a string",
                "messageId should not be empty",
              ],
            },
            error: {
              type: "string",
              example: "Bad Request",
            },
          },
        },
      },
    },
  })
  @ApiInternalServerErrorResponse({
    description:
      "Error in sending data to attestation manager. This error shows that attestation manager could not convert request to event or attestation manager could not send the event to the broker.",
    content: {
      "application/json": {
        schema: {
          type: "object",
          properties: {
            statusCode: {
              type: "number",
              example: 500,
            },
            message: {
              type: "string",
              example: "connect ECONNREFUSED 0.0.0.0.0:1234",
            },
          },
        },
      },
    },
  })
  @ApiOperation({
    summary: "Delete message by ID",
    description:
      "Method will delete a specific message by ID. The id of the response will be matched when you receive an event from the websocket",
    tags: ["Messages"],
  })
  async deleteMessage(@Param("id") messageId: string): Promise<GatewayAcceptedResponseDto> {
    const data = new IdReqDto();
    data.id = messageId;

    return this.amClient.sendPayload({
      pattern: "messages",
      payload: {
        source: "messages",
        data: data,
        type: MESSAGES_DELETE,
      },
    });
  }
}
