import { Injectable, Logger } from "@nestjs/common";
import { AgentService } from "../askar/agent.service";
import {
  ProofReqDto,
  CloudEventDto,
  CONNECTION_ACCEPT,
  CONNECTION_CREATE,
  CONNECTION_GET,
  CONNECTION_LIST,
  CONNECTION_DELETE,
  CreateCredentialDefinitionRequestDto,
  CreateInvitationResponseDto,
  CreateSchemaRequestDto,
  CredentialFilterDto,
  CRED_DEF_CREATE,
  CRED_DEF_LIST,
  CRED_DEF_GET,
  CRED_ISSUE,
  CRED_LIST,
  CRED_QUERY,
  CRED_GET,
  CRED_OFFER_ACCEPT,
  CRED_OFFER_DECLINE,
  CRED_DELETE,
  CRED_OFFER_LIST,
  IssueCredentialRequestDto,
  IssueProofRequestDto,
  ProofFilterDto,
  PROOF_ACCEPT,
  PROOF_DECLINE,
  PROOF_ISSUE,
  PROOF_LIST,
  PROOF_QUERY,
  PROOF_GET,
  PROOF_DELETE,
  SCHEMA_CREATE,
  SCHEMA_GET,
  SCHEMA_LIST,
  MakeBasicMessageRequestDto,
  MessageFilterDto,
  MESSAGES_MAKE,
  MESSAGES_QUERY,
  MESSAGES_DELETE,
  IdReqDto,
} from "@ocm-engine/dtos";
import asyncRetry from "async-retry";

@Injectable()
export class EventHandlerService {
  private readonly logger = new Logger(EventHandlerService.name);

  constructor(private readonly agentService: AgentService) {}

  async handle<T>(event: CloudEventDto<T>) {
    let data;
    let dto;
    await asyncRetry(
      async () => {
        switch (event.type) {
          case CONNECTION_CREATE:
            data = await this.agentService.createInvitation();
            break;
          case CONNECTION_ACCEPT:
            dto = event.data as CreateInvitationResponseDto;
            data = await this.agentService.acceptInvitation(dto.invitationUrl);
            break;

          case CONNECTION_LIST:
            data = await this.agentService.fetchConnections();
            break;

          case CONNECTION_GET:
            dto = event.data as IdReqDto;
            data = await this.agentService.getConnectionById(dto.id);
            break;

          case CONNECTION_DELETE:
            dto = event.data as IdReqDto;
            data = await this.agentService.deleteConnectionById(dto.id);
            break;

          case SCHEMA_CREATE:
            dto = event.data as CreateSchemaRequestDto;
            data = await this.agentService.createSchema(dto);
            break;

          case SCHEMA_LIST:
            data = await this.agentService.fetchSchemas();
            break;

          case SCHEMA_GET:
            dto = event.data as IdReqDto;
            data = await this.agentService.getSchemaById(dto.id);
            break;

          case CRED_DEF_CREATE:
            data = await this.agentService.createCredentialDefinition(
              event.data as CreateCredentialDefinitionRequestDto,
            );
            break;

          case CRED_DEF_LIST:
            data = await this.agentService.credentialDefinitions();
            break;

          case CRED_DEF_GET:
            dto = event.data as IdReqDto;
            data = await this.agentService.getCredentialDefinitionsById(dto.id);
            break;

          case CRED_ISSUE:
            data = await this.agentService.issueCredential(
              event.data as IssueCredentialRequestDto,
            );
            break;

          case CRED_LIST:
            data = await this.agentService.credentials();
            break;

          case CRED_QUERY:
            dto = event.data as CredentialFilterDto;
            data = await this.agentService.queryCredentials(
              dto
            );
            break;

          case CRED_GET:
            dto = event.data as IdReqDto;
            data = await this.agentService.getCredentialById(dto.id);
            break;

          case CRED_OFFER_LIST:
            data = await this.agentService.credentialByStatedOfferReceived();
            break;

          case CRED_OFFER_ACCEPT:
            dto = event.data as IdReqDto;
            data = await this.agentService.acceptCredential(
              dto.id,
            );
            break;

          case CRED_OFFER_DECLINE:
            dto = event.data as IdReqDto;
            data = await this.agentService.declineCredential(
              dto.id,
            );
            break;

          case CRED_DELETE:
            dto = event.data as IdReqDto;
            data = await this.agentService.deleteCredentialById(
              dto.id,
            );
            break;

          case PROOF_ISSUE:
            dto = event.data as IssueProofRequestDto;
            data = await this.agentService.issueProof(dto);
            break;

          case PROOF_LIST:
            data = await this.agentService.proofs();
            break;

          case PROOF_QUERY:
            dto = event.data as ProofFilterDto;
            data = await this.agentService.queryProofs(
              dto
            );
            break;

          case PROOF_GET:
            dto = event.data as IdReqDto;
            data = await this.agentService.getProofById(dto.id);
            break;

          case PROOF_ACCEPT:
            data = await this.agentService.acceptProof(
              event.data as ProofReqDto,
            );
            break;

          case PROOF_DECLINE:
            dto = event.data as IdReqDto;
            data = await this.agentService.declineProofRequest(
              dto.id,
            );
            break;

          case PROOF_DELETE:
            dto = event.data as IdReqDto;
            data = await this.agentService.deleteProofById(
              dto.id,
            );
            break;

          case MESSAGES_MAKE:
            dto = event.data as MakeBasicMessageRequestDto;
            data = await this.agentService.sendMessage(dto);
            break;

          case MESSAGES_QUERY:
            dto = event.data as MessageFilterDto;
            data = await this.agentService.queryBasicMessages(
              dto
            );
            break;

          case MESSAGES_DELETE:
            dto = event.data as IdReqDto;
            data = await this.agentService.deleteMessageById(
              dto.id,
            );
            break;
        }
      },
      {
        retries: 5,
        onRetry: (error: unknown) => {
          this.logger.log(JSON.stringify(error, null, 2));
        },
      },
    );

    event.data = data;
    return event;
  }
}
