import { Injectable } from "@nestjs/common";
import { AskarService } from "./askar.service";
import {
  ConnectionRecordDto,
  ProofReqDto,
  ConnectionNotFoundError,
  CreateCredentialDefinitionRequestDto,
  CreddefRecordDto,
  CreateInvitationResponseDto,
  CreateSchemaRequestDto,
  SchemaRecordDto,
  CredentialNotCreatedError,
  IssueCredentialRequestDto,
  CredentialRecordDto,
  IssueProofRequestDto,
  ProofRecordDto,
  MakeBasicMessageRequestDto,
  MessageRecordDto,
  SchemaNotCreatedError,
  SchemaNotFoundError,
  CredentialFilterDto,
  ProofFilterDto,
  MessageFilterDto,
  ProofIssueResponseDto,
  CredentialIssueResponseDto,
} from "@ocm-engine/dtos";
import {
  AutoAcceptCredential,
  BasicMessageRecord,
  BasicMessageRole,
  ConnectionRecord,
  CredentialExchangeRecord,
  CredentialState,
  ProofExchangeRecord,
  ProofState,
  Query,
} from "@aries-framework/core";
import { AnonCredsRequestedAttribute } from "@aries-framework/anoncreds";
import { uuid } from "@aries-framework/core/build/utils/uuid";
import { waitForProofExchangeRecordSubject } from "../agent.utils";

@Injectable()
export class AgentService {
  constructor(private readonly askar: AskarService) {}

  createInvitation = async () => {
    const outOfBoundRecord = await this.askar.agent.oob.createInvitation();

    const i = new CreateInvitationResponseDto();
    i.invitationUrl = outOfBoundRecord.outOfBandInvitation.toUrl({
      domain: this.askar.agentConfig.agentPeerAddress,
    });

    i.shortInvitationUrl = `${this.askar.agentConfig.agentPeerAddress}/invitations/${outOfBoundRecord.outOfBandInvitation.id}`;
    return i;
  };

  acceptInvitation = async (invitationUrl: string) => {
    const { connectionRecord } =
      await this.askar.agent.oob.receiveInvitationFromUrl(invitationUrl);

    if (typeof connectionRecord === "undefined") {
      throw new ConnectionNotFoundError();
    }

    const r = new ConnectionRecordDto();
    r.connectionName = connectionRecord.theirLabel;
    r.state = connectionRecord.state;
    r.id = connectionRecord.id;
    r.did = connectionRecord.did;
    r.invitationDid = connectionRecord.invitationDid;
    r.outOfBandId = connectionRecord.outOfBandId;
    r.createdAt = connectionRecord.createdAt;

    return r;
  };

  async fetchConnections() {
    const agentResponse = await this.askar.agent.connections.getAll();

    const connectionArray = agentResponse.map((singleConnectionRes) => {
      const connectionResponse = new ConnectionRecordDto();
      connectionResponse.id = singleConnectionRes.id;
      connectionResponse.state = singleConnectionRes.state;
      connectionResponse.connectionName = singleConnectionRes.theirLabel;
      connectionResponse.alias = singleConnectionRes.alias;
      connectionResponse.did = singleConnectionRes.did;
      connectionResponse.invitationDid = singleConnectionRes.invitationDid;
      connectionResponse.outOfBandId = singleConnectionRes.outOfBandId;
      connectionResponse.createdAt = singleConnectionRes.createdAt;

      return connectionResponse;
    });

    return connectionArray;
  }

  getConnectionById = async (id: string) => {
    const agentResponse = await this.askar.agent.connections.findById(id);

    if (!agentResponse) {
      throw new ConnectionNotFoundError();
    }

    const connectionResponse = new ConnectionRecordDto();
    connectionResponse.id = agentResponse.id;
    connectionResponse.state = agentResponse.state;
    connectionResponse.connectionName = agentResponse.theirLabel;
    connectionResponse.alias = agentResponse.alias;
    connectionResponse.did = agentResponse.did;
    connectionResponse.invitationDid = agentResponse.invitationDid;
    connectionResponse.outOfBandId = agentResponse.outOfBandId;
    connectionResponse.createdAt = agentResponse.createdAt;

    return connectionResponse;
  };

  deleteConnectionById = async (id: string): Promise<null> => {
    await this.askar.agent.connections.deleteById(id);

    return null;
  };

  fetchSchemas = async () => {
    const agentResponse =
      await this.askar.agent.modules.anoncreds.getCreatedSchemas({});

    const schemaResponses = agentResponse.map((singleSchemaRes) => {
      const schemaResponse = new SchemaRecordDto();
      schemaResponse.id = singleSchemaRes.schemaId;
      schemaResponse.name = singleSchemaRes.schema.name;
      schemaResponse.attributes = singleSchemaRes.schema.attrNames;
      schemaResponse.version = singleSchemaRes.schema.version;
      schemaResponse.issuerId = singleSchemaRes.schema.issuerId;
      schemaResponse.methodName = singleSchemaRes.methodName;

      return schemaResponse;
    });

    return schemaResponses;
  };

  getSchemaById = async (schemaId: string) => {
    const agentResponse = await this.askar.agent.modules.anoncreds.getSchema(
      schemaId,
    );

    if (!agentResponse || !agentResponse.schema) {
      throw new SchemaNotFoundError();
    }

    const schemaResponse = new SchemaRecordDto();
    schemaResponse.id = agentResponse.schemaId;
    schemaResponse.name = agentResponse.schema.name;
    schemaResponse.attributes = agentResponse.schema.attrNames;
    schemaResponse.version = agentResponse.schema.version;
    schemaResponse.issuerId = agentResponse.schema.issuerId;

    return schemaResponse;
  };

  createSchema = async (schema: CreateSchemaRequestDto) => {
    const dids = await this.askar.agent.dids.getCreatedDids({ method: "indy" });

    const schemaResult =
      await this.askar.agent.modules.anoncreds.registerSchema({
        schema: {
          name: schema.name,
          issuerId: dids[0].did,
          attrNames: schema.attributes,
          version: schema.version,
        },
        options: {},
      });

    if (schemaResult.schemaState.state !== "finished") {
      throw new SchemaNotCreatedError();
    }

    const response = new SchemaRecordDto();

    response.name = schemaResult.schemaState.schema.name;
    response.id = schemaResult.schemaState.schemaId;
    response.issuerId = schemaResult.schemaState.schema.issuerId;
    response.version = schemaResult.schemaState.schema.version;
    response.attributes = schemaResult.schemaState.schema.attrNames;

    return response;
  };

  credentialDefinitions = async () => {
    const credentialDefinitions =
      await this.askar.agent.modules.anoncreds.getCreatedCredentialDefinitions({ });

    const response: Array<CreddefRecordDto> = [];
    for (const credDef of credentialDefinitions) {
      const cd = new CreddefRecordDto();
      cd.id = credDef.credentialDefinitionId;
      cd.schemaId = credDef.credentialDefinition.schemaId;
      cd.issuerId = credDef.credentialDefinition.issuerId;
      cd.tag = credDef.credentialDefinition.tag;
      response.push(cd);
    }

    return response;
  };

  getCredentialDefinitionsById = async (credentialDefinitionId: string) => {
    const credDefs = await this.askar.agent.modules.anoncreds
      .getCreatedCredentialDefinitions({ credentialDefinitionId });

    const credDef = credDefs[0] || null;
    if (!credDef) {
      return null;
    }

    const cd = new CreddefRecordDto();
    cd.id = credDef.credentialDefinitionId;
    cd.schemaId = credDef.credentialDefinition.schemaId;
    cd.issuerId = credDef.credentialDefinition.issuerId;
    cd.tag = credDef.credentialDefinition.tag;

    return cd;
  };

  createCredentialDefinition = async (
    credentialDefinitionDto: CreateCredentialDefinitionRequestDto,
  ) => {
    const dids = await this.askar.agent.dids.getCreatedDids({ method: "indy" });

    const credDef =
      await this.askar.agent.modules.anoncreds.registerCredentialDefinition({
        credentialDefinition: {
          tag: credentialDefinitionDto.tag,
          issuerId: dids[0].did,
          schemaId: credentialDefinitionDto.schemaId,
        },
        options: {},
      });

    if (credDef.credentialDefinitionState.state !== "finished") {
      throw new CredentialNotCreatedError();
    }

    const response = new CreddefRecordDto();
    response.id = credDef.credentialDefinitionState.credentialDefinitionId;
    response.schemaId =
      credDef.credentialDefinitionState.credentialDefinition.schemaId;
    response.issuerId =
      credDef.credentialDefinitionState.credentialDefinition.issuerId;
    response.tag = credDef.credentialDefinitionState.credentialDefinition.tag;

    return response;
  };

  issueCredential = async (
    issueCredentialDto: IssueCredentialRequestDto,
  ): Promise<CredentialIssueResponseDto> => {
    console.log(
      "Incoming request",
      JSON.stringify(issueCredentialDto, null, 2),
    );

    if (!issueCredentialDto.connectionId) {
      try {
        const { credentialRecord, message } =
          await this.askar.agent.credentials.createOffer({
            protocolVersion: "v2",
            credentialFormats: {
              anoncreds: {
                credentialDefinitionId: issueCredentialDto.credentialDefinitionId,
                attributes: issueCredentialDto.attributes,
              },
            },
            autoAcceptCredential: AutoAcceptCredential.ContentApproved,
          });

        const outOfBandRecord = await this.askar.agent.oob.createInvitation({
          messages: [message],
          handshake: false,
        });

        // const credentialUrl = outOfBandRecord.outOfBandInvitation.toUrl({
        //   domain: this.askar.agentConfig.agentPeerAddress,
        // });

        const shortCredentialUrl = `${this.askar.agentConfig.agentPeerAddress}/invitations/${outOfBandRecord.outOfBandInvitation.id}`;

        const dto = new CredentialRecordDto();
        dto.id = credentialRecord.id;
        dto.state = credentialRecord.state;
        dto.connectionId = credentialRecord.connectionId;
        dto.attributes = credentialRecord.credentialAttributes;
        dto.createdAt = credentialRecord.createdAt;

        return {
          credentialUrl: null,
          shortCredentialUrl: shortCredentialUrl,
          credentialRecord: dto,
        };
      } catch (e) {
        console.log("------------------------");
        console.log(JSON.stringify(e, null, 2));
        console.log("------------------------");
        throw new Error("unknown");
      }
    }
    try {
      const credentialExchangeRecord =
        await this.askar.agent.credentials.offerCredential({
          protocolVersion: "v2",
          connectionId: issueCredentialDto.connectionId,
          credentialFormats: {
            anoncreds: {
              credentialDefinitionId: issueCredentialDto.credentialDefinitionId,
              attributes: issueCredentialDto.attributes,
            },
          },
        });

      const dto = new CredentialRecordDto();
      dto.id = credentialExchangeRecord.id;
      dto.state = credentialExchangeRecord.state;
      dto.connectionId = credentialExchangeRecord.connectionId;
      dto.attributes = credentialExchangeRecord.credentialAttributes;
      dto.createdAt = credentialExchangeRecord.createdAt;

      return {
        credentialUrl: null,
        shortCredentialUrl: null,
        credentialRecord: dto,
      };
    } catch (e) {
      console.log("------------------------");
      console.log(JSON.stringify(e, null, 2));
      console.log("------------------------");
      throw new Error("unknown");
    }
  };

  acceptCredential = async (credentialRecordId: string) => {
    const linkSecretIds =
      await this.askar.agent.modules.anoncreds.getLinkSecretIds();

    if (linkSecretIds.length === 0) {
      await this.askar.agent.modules.anoncreds.createLinkSecret();
    }

    const credentialExchangeRecord =
      await this.askar.agent.credentials.acceptOffer({
        credentialRecordId,
      });

    const response = new CredentialRecordDto();
    response.id = credentialExchangeRecord.id;
    response.state = credentialExchangeRecord.state;
    response.connectionId = credentialExchangeRecord.connectionId;
    response.attributes = credentialExchangeRecord.credentialAttributes;
    response.createdAt = credentialExchangeRecord.createdAt;

    return response;
  };

  declineCredential = async (credentialRecordId: string) => {
    const linkSecretIds =
      await this.askar.agent.modules.anoncreds.getLinkSecretIds();

    if (linkSecretIds.length === 0) {
      await this.askar.agent.modules.anoncreds.createLinkSecret();
    }

    const credentialExchangeRecord =
      await this.askar.agent.credentials.declineOffer(credentialRecordId);

    const response = new CredentialRecordDto();
    response.id = credentialExchangeRecord.id;
    response.state = credentialExchangeRecord.state;
    response.connectionId = credentialExchangeRecord.connectionId;
    response.attributes = credentialExchangeRecord.credentialAttributes;
    response.createdAt = credentialExchangeRecord.createdAt;

    return response;
  };

  queryCredentials = async (filter: CredentialFilterDto) => {
    const query: Query<CredentialExchangeRecord>[] = [];

    if (filter.states) {
      const stateQuery: Query<CredentialExchangeRecord> = {
        $or: filter.states.map((state) => ({ state })),
      };
      query.push(stateQuery);
    }

    if (filter.connectionId) {
      const connectionQuery: Query<CredentialExchangeRecord> = {
        connectionId: filter.connectionId,
      };
      query.push(connectionQuery);
    }

    const credentials = await this.askar.agent.credentials.findAllByQuery({
      $and: query,
    });

    const response: Array<CredentialRecordDto> = [];
    for (const offer of credentials) {
      const t = new CredentialRecordDto();
      t.id = offer.id;
      t.state = offer.state;
      t.connectionId = offer.connectionId;
      t.createdAt = offer.createdAt;
      t.attributes = offer.credentialAttributes;
      response.push(t);
    }

    return response;
  };

  credentials = async () => {
    const credentials = await this.askar.agent.credentials.findAllByQuery({
      state: CredentialState.Done,
    });
    // REVIEW: get All credentials, or make the filter optional?
    // const credentials = await this.askar.agent.credentials.getAll();

    const response: Array<CredentialRecordDto> = [];
    for (const offer of credentials) {
      const t = new CredentialRecordDto();
      t.id = offer.id;
      t.state = offer.state;
      t.connectionId = offer.connectionId;
      t.createdAt = offer.createdAt;
      t.attributes = offer.credentialAttributes;
      response.push(t);
    }

    return response;
  };

  getCredentialById = async (credentialId: string) => {
    const credentialRecord = await this.askar.agent.credentials.findById(
      credentialId,
    );

    if (!credentialRecord) {
      return null;
    }

    const credential = new CredentialRecordDto();
    credential.id = credentialRecord.id;
    credential.state = credentialRecord.state;
    credential.connectionId = credentialRecord.connectionId;
    credential.createdAt = credentialRecord.createdAt;
    credential.attributes = credentialRecord.credentialAttributes;

    return credential;
  };

  credentialByStatedOfferReceived = async () => {
    const offers = await this.askar.agent.credentials.findAllByQuery({
      state: CredentialState.OfferReceived,
    });

    const response: Array<CredentialRecordDto> = [];
    for (const offer of offers) {
      const t = new CredentialRecordDto();
      t.id = offer.id;
      t.state = offer.state;
      t.connectionId = offer.connectionId;
      t.createdAt = offer.createdAt;
      t.attributes = offer.credentialAttributes;

      response.push(t);
    }

    return response;
  };

  issueProof = async (
    issueProofDto: IssueProofRequestDto,
  ): Promise<ProofIssueResponseDto> => {
    console.log(JSON.stringify(issueProofDto, null, 2));
    const requestedAttributes: Record<string, AnonCredsRequestedAttribute> = {};

    for (const attr of issueProofDto.attributes) {
      requestedAttributes[uuid()] = {
        name: attr.attributeName,
        restrictions: [
          {
            schema_id: attr.schemaId,
            cred_def_id: attr.credentialDefinitionId,
          },
        ],
      };
    }

    if (!issueProofDto.connectionId) {
      console.log("connection Id not detected, creating oob proof");
      const { proofRecord, message } =
        await this.askar.agent.proofs.createRequest({
          protocolVersion: "v2",
          proofFormats: {
            anoncreds: {
              name: "proof-request",
              version: "1.0",
              requested_attributes: requestedAttributes,
            },
          },
        });

      const outOfBandRecord = await this.askar.agent.oob.createInvitation({
        messages: [message],
        handshake: false,
      });

      const proofUrl = outOfBandRecord.outOfBandInvitation.toUrl({
        domain: this.askar.agentConfig.agentPeerAddress,
      });
      const shortProofUrl = `${this.askar.agentConfig.agentPeerAddress}/invitations/${outOfBandRecord.outOfBandInvitation.id}`;

      const dto = new ProofRecordDto();
      dto.id = proofRecord.id;
      dto.connectionId = proofRecord.connectionId;
      dto.state = proofRecord.state;
      dto.updatedAt = proofRecord.updatedAt;
      dto.createdAt = proofRecord.createdAt;

      return {
        proofUrl: proofUrl,
        shortProofUrl: shortProofUrl,
        proofRecord: dto,
      };
    }

    console.log(`${issueProofDto.connectionId} detected, issuing proof`);

    const exchangeRecord = await this.askar.agent.proofs.requestProof({
      protocolVersion: "v2",
      connectionId: issueProofDto.connectionId,
      proofFormats: {
        anoncreds: {
          name: "proof-request",
          version: "1.0",
          requested_attributes: requestedAttributes,
        },
      },
    });

    const response = new ProofRecordDto();
    response.id = exchangeRecord.id;
    response.connectionId = exchangeRecord.connectionId;
    response.state = exchangeRecord.state;
    response.updatedAt = exchangeRecord.updatedAt;
    response.createdAt = exchangeRecord.createdAt;

    return {
      proofUrl: null,
      shortProofUrl: null,
      proofRecord: response,
    };
  };

  deleteCredentialById = async (id: string): Promise<null> => {
    await this.askar.agent.credentials.deleteById(id);

    return null;
  };

  queryProofs = async (filter: ProofFilterDto) => {
    const query: Query<ProofExchangeRecord>[] = [];

    if (filter.isVerified !== null && filter.isVerified !== undefined) {
      const isVerifiedQuery: Query<ProofExchangeRecord> = {
        isVerified: filter.isVerified,
      };
      query.push(isVerifiedQuery);
    }

    if (filter.states) {
      const stateQuery: Query<ProofExchangeRecord> = {
        $or: filter.states.map((state) => ({ state })),
      };
      query.push(stateQuery);
    }

    if (filter.connectionId) {
      const connectionQuery: Query<ProofExchangeRecord> = {
        connectionId: filter.connectionId,
      };
      query.push(connectionQuery);
    }

    const proofs = await this.askar.agent.proofs.findAllByQuery({
      $and: query,
    });

    const response: Array<ProofRecordDto> = [];
    for (const proof of proofs) {
      const t = new ProofRecordDto();
      t.id = proof.id;
      t.connectionId = proof.connectionId;
      t.state = proof.state;
      t.updatedAt = proof.updatedAt;
      t.createdAt = proof.createdAt;

      response.push(t);
    }

    return response;
  };

  proofs = async () => {
    const proofs = await this.askar.agent.proofs.findAllByQuery({
      state: ProofState.RequestReceived,
    });

    const response: Array<ProofRecordDto> = [];
    for (const proof of proofs) {
      const t = new ProofRecordDto();
      t.id = proof.id;
      t.connectionId = proof.connectionId;
      t.state = proof.state;
      t.updatedAt = proof.updatedAt;
      t.createdAt = proof.createdAt;

      response.push(t);
    }

    return response;
  };

  getProofById = async (proofRecordId: string) => {
    const proofRecord = await this.askar.agent.proofs.findById(proofRecordId);

    if (!proofRecord) {
      return null;
    }

    if (proofRecord.state === ProofState.Done) {
      const data = await this.askar.agent.proofs.getFormatData(proofRecord.id);
      console.log(JSON.stringify(data, null, 2));
    }

    const proofResponse = new ProofRecordDto();

    proofResponse.id = proofRecord.id;
    proofResponse.connectionId = proofRecord.connectionId;
    proofResponse.state = proofRecord.state;
    proofResponse.updatedAt = proofRecord.updatedAt;
    proofResponse.createdAt = proofRecord.createdAt;

    return proofResponse;
  };

  acceptProof = async (acceptProofDto: ProofReqDto) => {
    if (acceptProofDto.proofUrl) {
      return this.acceptOobProof(acceptProofDto.proofUrl);
    }
    return this.acceptConnectionProof(acceptProofDto.proofId);
  };

  acceptOobProof = async (url: string) => {
    await this.askar.agent.oob.receiveInvitationFromUrl(url, {
      autoAcceptConnection: false,
      autoAcceptInvitation: true,
      // reuseConnection: true,
    });

    const record = await waitForProofExchangeRecordSubject(this.askar.agentR, {
      state: ProofState.RequestReceived,
    });

    const requestedCredentials =
      await this.askar.agent.proofs.selectCredentialsForRequest({
        proofRecordId: record.id,
      });

    const acceptedRecord = await this.askar.agent.proofs.acceptRequest({
      proofRecordId: record.id,
      proofFormats: requestedCredentials.proofFormats,
    });

    const response = new ProofRecordDto();

    response.id = acceptedRecord.id;
    response.state = acceptedRecord.state;
    response.updatedAt = acceptedRecord.updatedAt;
    response.createdAt = acceptedRecord.createdAt;

    return response;
  };

  acceptConnectionProof = async (proofRecordId: string) => {
    console.log(`accepting proof request for ${proofRecordId}`);
    const requestedCredentials =
      await this.askar.agent.proofs.selectCredentialsForRequest({
        proofRecordId,
      });

    console.log(JSON.stringify(requestedCredentials, null, 2));

    const proof = await this.askar.agent.proofs.acceptRequest({
      proofRecordId,
      proofFormats: requestedCredentials.proofFormats,
    });

    console.log(JSON.stringify(proof, null, 2));

    const response = new ProofRecordDto();
    response.id = proof.id;
    response.connectionId = proof.connectionId;
    response.state = proof.state;
    response.updatedAt = proof.updatedAt;
    response.createdAt = proof.createdAt;

    return response;
  };

  declineProofRequest = async (proofRecordId: string) => {
    const resultFromDecline = await this.askar.agent.proofs.declineRequest({
      proofRecordId,
      // sendProblemReport: false, // REVIEW: do we have a use case for this key?
    });

    const declineResponse = new ProofRecordDto();
    declineResponse.id = resultFromDecline.id;
    declineResponse.connectionId = resultFromDecline.connectionId;
    declineResponse.state = resultFromDecline.state;
    declineResponse.updatedAt = resultFromDecline.updatedAt;
    declineResponse.createdAt = resultFromDecline.createdAt;

    return declineResponse;
  };

  deleteProofById = async (id: string): Promise<null> => {
    await this.askar.agent.proofs.deleteById(id);

    return null;
  };

  resolve = async (did: string) => {
    return this.askar.agent.dids.resolve(did);
  };

  queryBasicMessages = async (filter: MessageFilterDto) => {
    const query: Query<BasicMessageRecord>[] = [];

    if (filter.role) {
      const roleQuery: Query<BasicMessageRecord> = {
        role: filter.role,
      };
      query.push(roleQuery);
    }

    if (filter.connectionId) {
      const connectionQuery: Query<BasicMessageRecord> = {
        connectionId: filter.connectionId,
      };
      query.push(connectionQuery);
    }

    const messages = await this.askar.agent.basicMessages.findAllByQuery({
      $and: query,
    });

    const connectionIds = messages.map((message) => message.connectionId);

    const connections = await this.askar.agent.connections.findAllByQuery({
      $or: connectionIds.map((p) => ({ id: p })),
    });
    const grouppedConnections = connections.reduce(
      (acc: { [connId: string]: ConnectionRecord }, conn) => {
        acc[conn.id] = conn;
        return acc;
      },
      {},
    );

    const response: Array<MessageRecordDto> = [];
    for (const message of messages) {
      const connection = grouppedConnections[message.connectionId];
      const label = connection?.theirLabel || "";

      const t = new MessageRecordDto();
      t.id = message.id;
      t.createdAt = message.createdAt;
      t.updatedAt = message.updatedAt;
      t.connectionId = message.connectionId;
      t.role = message.role;
      t.content = message.content;
      t.sentTime = message.sentTime;
      t.from = message.role === BasicMessageRole.Receiver ? label : "";
      t.to = message.role === BasicMessageRole.Sender ? label : "";

      response.push(t);
    }

    return response;
  };

  sendMessage = async (dto: MakeBasicMessageRequestDto) => {
    const messageRecord = await this.askar.agent.basicMessages.sendMessage(
      dto.connectionId,
      dto.content,
    );

    const connRecord = await this.askar.agent.connections.findById(
      dto.connectionId,
    );

    const response = new MessageRecordDto();
    response.id = messageRecord.id;
    response.createdAt = messageRecord.createdAt;
    response.updatedAt = messageRecord.updatedAt;
    response.connectionId = messageRecord.connectionId;
    response.role = messageRecord.role;
    response.content = messageRecord.content;
    response.sentTime = messageRecord.sentTime;
    response.from = "";
    response.to = connRecord?.theirLabel || "";

    return response;
  };

  deleteMessageById = async (id: string): Promise<null> => {
    await this.askar.agent.basicMessages.deleteById(id);

    return null;
  };
}
